@@ -22,7 +22,6 @@
 #include "panel-gconf.h"
 #include "panel-config-global.h"
 #include "panel-applet-frame.h"
-#include "egg-screen-exec.h"
 #include "panel-action-button.h"
 #include "panel-menu-bar.h"
 #include "panel-compatibility.h"
@@ -32,9 +31,13 @@
 #include "panel-menu-button.h"
 #include "panel-globals.h"
 #include "panel-properties-dialog.h"
+#include "panel-lockdown.h"
 
 #define SMALL_ICON_SIZE 20
 
+static GtkWidget *panel_applet_create_menu (AppletInfo  *info,
+					    PanelWidget *panel_widget);
+
 static GSList *registered_applets = NULL;
 static GSList *queued_position_saves = NULL;
 static guint   queued_position_source = 0;
@@ -141,11 +144,35 @@ panel_applet_clean (AppletInfo *info)
 	}
 }
 
+static void
+panel_applet_recreate_menu (AppletInfo	*info)
+{
+	GList *l;
+
+	if (!info->menu)
+		return;
+
+	for (l = info->user_menu; l; l = l->next) {
+		AppletUserMenu *menu = l->data;
+
+		menu->menuitem =NULL;
+		menu->submenu =NULL;
+	}
+
+	gtk_widget_unref (info->menu);
+	info->menu = panel_applet_create_menu (info,
+					       PANEL_WIDGET (info->widget->parent));
+}
+
 static void
 applet_remove_callback (GtkWidget  *widget,
 			AppletInfo *info)
 {
-	panel_profile_delete_object (info);
+
+	if (info->type == PANEL_OBJECT_DRAWER)
+		drawer_query_deletion (info->data);
+	else
+		panel_profile_delete_object (info);
 }
 
 static inline GdkScreen *
@@ -173,9 +200,6 @@ applet_callback_callback (GtkWidget      *widget,
 		if (!strcmp (menu->name, "properties"))
 			launcher_properties (
 				menu->info->data, screen);
-
-		else if (!strcmp (menu->name, "help_on_app"))
-			launcher_show_help (menu->info->data, screen);
 		break;
 	case PANEL_OBJECT_DRAWER: 
 		if (strcmp (menu->name, "properties")==0) {
@@ -239,80 +263,31 @@ panel_applet_get_callback (GList      *user_menu,
 }
 
 void
-panel_applet_add_callback (AppletInfo *info,
-			   const char *callback_name,
-			   const char *stock_item,
-			   const char *menuitem_text)
+panel_applet_add_callback (AppletInfo          *info,
+			   const char          *callback_name,
+			   const char          *stock_item,
+			   const char          *menuitem_text,
+			   CallbackEnabledFunc  is_enabled_func)
 {
 	AppletUserMenu *menu;
 
 	g_return_if_fail (info != NULL);
+	g_return_if_fail (panel_applet_get_callback (info->user_menu,
+						     callback_name) == NULL);
 
-	menu = panel_applet_get_callback (info->user_menu, callback_name);
-	if (menu == NULL) {
-		menu = g_new0 (AppletUserMenu, 1);
-		menu->name = g_strdup (callback_name);
-		menu->stock_item = g_strdup (stock_item);
-		menu->text = g_strdup (menuitem_text);
-		menu->sensitive = TRUE;
-		menu->info = info;
-		menu->menuitem = NULL;
-		menu->submenu = NULL;
-		info->user_menu = g_list_append (info->user_menu, menu);
-	} else {
-		g_free (menu->stock_item);
-		menu->stock_item = NULL;
-		g_free (menu->text);
-		menu->text = NULL;
-		menu->text = g_strdup (menuitem_text);
-		menu->stock_item = g_strdup (stock_item);
-	}
+	menu                  = g_new0 (AppletUserMenu, 1);
+	menu->name            = g_strdup (callback_name);
+	menu->stock_item      = g_strdup (stock_item);
+	menu->text            = g_strdup (menuitem_text);
+	menu->is_enabled_func = is_enabled_func;
+	menu->sensitive       = TRUE;
+	menu->info            = info;
+	menu->menuitem        = NULL;
+	menu->submenu         = NULL;
 
-	/*make sure the menu is rebuilt*/
-	if(info->menu) {
-		GList *list;
-		for(list=info->user_menu;list!=NULL;list=g_list_next(list)) {
-			AppletUserMenu *menu = list->data;
-			menu->menuitem=NULL;
-			menu->submenu=NULL;
-		}
-		gtk_widget_unref(info->menu);
-		info->menu = NULL;
-	}
-}
+	info->user_menu = g_list_append (info->user_menu, menu);
 
-void
-panel_applet_remove_callback (AppletInfo *info,
-			      const char *callback_name)
-{
-	AppletUserMenu *menu;
-
-	g_return_if_fail (info != NULL);
-	
-	menu = panel_applet_get_callback (info->user_menu, callback_name);
-	if (menu != NULL) {
-		info->user_menu = g_list_remove (info->user_menu, menu);
-		g_free(menu->name);
-		menu->name = NULL;
-		g_free(menu->stock_item);
-		menu->stock_item = NULL;
-		g_free(menu->text);
-		menu->text = NULL;
-		g_free(menu);
-	} else
-		return; /*it just isn't there*/
-
-	/*make sure the menu is rebuilt*/
-	if (info->menu != NULL) {
-		GList *list;
-		for (list = info->user_menu; list != NULL; list = list->next) {
-			AppletUserMenu *menu = list->data;
-			menu->menuitem = NULL;
-			menu->submenu = NULL;
-		}
-		gtk_widget_unref (info->menu);
-		info->menu = NULL;
-	}
+	panel_applet_recreate_menu (info);
 }
 
 static void
@@ -433,6 +408,7 @@ panel_applet_create_menu (AppletInfo  *info,
 	GtkWidget *menu;
 	GtkWidget *menuitem;
 	GList     *l;
+	gboolean   added_anything = FALSE;
 
 	menu = g_object_ref (panel_create_menu ());
 	gtk_object_sink (GTK_OBJECT (menu));
@@ -443,32 +419,36 @@ panel_applet_create_menu (AppletInfo  *info,
 	g_signal_connect (menu, "deactivate",
 			  G_CALLBACK (applet_menu_deactivate), info);
 
-	if (info->user_menu) {
-		for (l = info->user_menu; l; l = l->next) {
-			AppletUserMenu *user_menu = (AppletUserMenu *)l->data;
+	for (l = info->user_menu; l; l = l->next) {
+		AppletUserMenu *user_menu = l->data;
 
-			add_to_submenus (info, "", user_menu->name, user_menu, 
-					 menu, info->user_menu);
-		}
+		if (user_menu->is_enabled_func && !user_menu->is_enabled_func ())
+			continue;
+
+		add_to_submenus (info, "", user_menu->name, user_menu, 
+				 menu, info->user_menu);
+
+		added_anything = TRUE;
 	}
 
-	if (!commie_mode) {
+	if (!panel_lockdown_get_locked_down ()) {
 		GtkWidget *image;
 		gboolean   locked;
 		gboolean   lockable;
 		gboolean   movable;
 		gboolean   removable;
 
-
 		lockable = panel_applet_lockable (info);
 		movable = panel_applet_can_freely_move (info);
 		removable = panel_profile_list_is_writable (PANEL_GCONF_OBJECTS);
 
 		locked = panel_widget_get_applet_locked (panel_widget, info->widget);
 
-		menuitem = gtk_separator_menu_item_new ();
-		gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
-		gtk_widget_show (menuitem);
+		if (added_anything) {
+			menuitem = gtk_separator_menu_item_new ();
+			gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
+			gtk_widget_show (menuitem);
+		}
 
 		menuitem = gtk_image_menu_item_new ();
 		image = gtk_image_new_from_stock (GTK_STOCK_REMOVE,
@@ -506,6 +486,13 @@ panel_applet_create_menu (AppletInfo  *info,
 		info->move_item = menuitem;
 		g_object_add_weak_pointer (G_OBJECT (menuitem),
 					   (gpointer *) &info->move_item);
+
+		added_anything = TRUE;
+	}
+
+	if ( ! added_anything) {
+		g_object_unref (menu);
+		return NULL;
 	}
 
 	return menu;
@@ -544,10 +531,11 @@ applet_show_menu (AppletInfo     *info,
 
 	panel_widget = PANEL_WIDGET (info->widget->parent);
 
-	if (!info->menu)
+	if (info->menu == NULL)
 		info->menu = panel_applet_create_menu (info, panel_widget);
 
-	g_assert (info->menu);
+	if (info->menu == NULL)
+		return;
 
 	panel_toplevel_block_auto_hide (panel_widget->toplevel);
 
@@ -639,8 +627,11 @@ panel_applet_destroy (GtkWidget  *widget,
 		}
 	}
 
+	panel_lockdown_notify_remove (G_CALLBACK (panel_applet_recreate_menu),
+				      info);
+
 	if (info->menu)
-		gtk_widget_unref (info->menu);
+		g_object_unref (info->menu);
 	info->menu = NULL;
 
 	if (info->data_destroy)
@@ -1067,6 +1058,9 @@ panel_applet_register (GtkWidget       *applet,
 
 	g_object_set_data (G_OBJECT (applet), "applet_info", info);
 
+	panel_lockdown_notify_add (G_CALLBACK (panel_applet_recreate_menu),
+				   info);
+
 	if (type == PANEL_OBJECT_DRAWER) {
 		Drawer *drawer = data;
 		PanelWidget *assoc_panel;
@@ -1126,9 +1120,7 @@ panel_applet_register (GtkWidget       *applet,
 
 	orientation_change (info, panel);
 	size_change (info, panel);
-#ifdef FIXME_FOR_NEW_CONFIG
 	back_change (info, panel);
-#endif
 
 	if (type != PANEL_OBJECT_BONOBO)
 		gtk_widget_grab_focus (applet);
@@ -1151,35 +1143,60 @@ panel_applet_get_position (AppletInfo *applet)
 	return applet_data->pos;
 }
 
-/* True if all the keys relevant to moving are writable
-   (position, toplevel_id, panel_right_stick) */
 gboolean
 panel_applet_can_freely_move (AppletInfo *applet)
 {
-	GConfClient *client = panel_gconf_get_client ();
-	const char *profile = panel_profile_get_name ();
-	PanelGConfKeyType key_type = applet->type == PANEL_OBJECT_BONOBO ? PANEL_GCONF_APPLETS : PANEL_GCONF_OBJECTS;
-	const char *key;
+	PanelWidget       *panel_widget;
+	GConfClient       *client;
+	PanelGConfKeyType  key_type;
+	const char        *profile;
+	const char        *key;
+
+	if (panel_lockdown_get_locked_down ())
+		return FALSE;
+
+	panel_widget = PANEL_WIDGET (applet->widget->parent);
+
+	client  = panel_gconf_get_client ();
+	profile = panel_profile_get_name ();
+	
+	key_type = (applet->type == PANEL_OBJECT_BONOBO) ? PANEL_GCONF_APPLETS : PANEL_GCONF_OBJECTS;
        
 	key = panel_gconf_full_key (key_type, profile, applet->id, "position");
-	if ( ! gconf_client_key_is_writable (client, key, NULL))
+	if (!gconf_client_key_is_writable (client, key, NULL))
 		return FALSE;
+
 	key = panel_gconf_full_key (key_type, profile, applet->id, "toplevel_id");
-	if ( ! gconf_client_key_is_writable (client, key, NULL))
+	if (!gconf_client_key_is_writable (client, key, NULL))
 		return FALSE;
+
 	key = panel_gconf_full_key (key_type, profile, applet->id, "panel_right_stick");
-	if ( ! gconf_client_key_is_writable (client, key, NULL))
+	if (!gconf_client_key_is_writable (client, key, NULL))
 		return FALSE;
+
 	return TRUE;
 }
 
-/* True if the locked flag is writable */
 gboolean
 panel_applet_lockable (AppletInfo *applet)
 {
-	GConfClient *client = panel_gconf_get_client ();
-	const char *profile = panel_profile_get_name ();
-	PanelGConfKeyType key_type = applet->type == PANEL_OBJECT_BONOBO ? PANEL_GCONF_APPLETS : PANEL_GCONF_OBJECTS;
-	const char *key = panel_gconf_full_key (key_type, profile, applet->id, "locked");
+	GConfClient        *client;
+	PanelWidget        *panel_widget;
+	PanelGConfKeyType   key_type;
+	const char         *profile;
+	const char         *key;
+
+	if (panel_lockdown_get_locked_down ())
+		return FALSE;
+	
+	panel_widget = PANEL_WIDGET (applet->widget->parent);
+
+	client  = panel_gconf_get_client ();
+	profile = panel_profile_get_name ();
+	
+	key_type = (applet->type == PANEL_OBJECT_BONOBO) ? PANEL_GCONF_APPLETS : PANEL_GCONF_OBJECTS;
+
+	key = panel_gconf_full_key (key_type, profile, applet->id, "locked");
+
 	return gconf_client_key_is_writable (client, key, NULL);
 }
