@@ -29,6 +29,7 @@
 #include "xstuff.h"
 #include "panel-globals.h"
 #include "panel-stock-icons.h"
+#include "panel-lockdown.h"
 
 static void
 drawer_click (GtkWidget *w, Drawer *drawer)
@@ -341,6 +342,8 @@ create_drawer_applet (PanelToplevel    *toplevel,
 	if (tooltip && tooltip [0]) {
 		panel_toplevel_set_name (toplevel, tooltip);
 		gtk_tooltips_set_tip (panel_tooltips, drawer->button, tooltip, NULL);
+	} else {
+		panel_toplevel_set_name (toplevel, _("Drawer"));
 	}
 	
 	gtk_drag_dest_set (drawer->button, 0, NULL, 0, 0); 
@@ -590,14 +593,17 @@ load_drawer_applet (char          *toplevel_id,
 	panel_widget_set_applet_expandable (panel_widget, GTK_WIDGET (drawer->button), FALSE, TRUE);
 	panel_widget_set_applet_size_constrained (panel_widget, GTK_WIDGET (drawer->button), TRUE);
 
-	if ( ! panel_profile_get_locked_down ())
-		panel_applet_add_callback (drawer->info,
-					   "properties",
-					   GTK_STOCK_PROPERTIES,
-					   _("_Properties"));
+	panel_applet_add_callback (drawer->info,
+				   "properties",
+				   GTK_STOCK_PROPERTIES,
+				   _("_Properties"),
+				   panel_lockdown_get_locked_down);
 
-	panel_applet_add_callback (
-		drawer->info, "help", GTK_STOCK_HELP, _("_Help"));
+	panel_applet_add_callback (drawer->info,
+				   "help",
+				   GTK_STOCK_HELP,
+				   _("_Help"),
+				   NULL);
 
 	panel_drawer_connect_to_gconf (drawer);
 }
@@ -797,10 +803,10 @@ drawer_query_deletion (Drawer *drawer)
 			  	  G_CALLBACK (drawer_deletion_response),
 			  	  drawer);
 
-		panel_signal_connect_object_while_alive (
-				G_OBJECT (drawer->toplevel), "destroy",
-				G_CALLBACK (gtk_widget_destroy),
-				G_OBJECT (dialog));
+		g_signal_connect_object (drawer->toplevel, "destroy",
+					 G_CALLBACK (gtk_widget_destroy),
+					 dialog,
+					 G_CONNECT_SWAPPED);
 
         	gtk_widget_show_all (dialog);
 	}
