@@ -38,12 +38,11 @@
 #include "panel-toplevel.h"
 #include "panel-a11y.h"
 #include "panel-globals.h"
+#include "panel-multiscreen.h"
+#include "panel-lockdown.h"
 
 #include "quick-desktop-reader.h"
 
-#include "egg-screen-exec.h"
-#include "egg-screen-url.h"
-
 #undef LAUNCHER_DEBUG
 
 static void properties_apply (Launcher *launcher);
@@ -133,7 +132,7 @@ launch_url (Launcher *launcher)
 		return;
 	}
 
-	egg_url_show_on_screen (url, screen, &error);
+	gnome_url_show_on_screen (url, screen, &error);
 	if (error) {
 		panel_error_dialog (screen,
 				    "cannot_show_url_dialog",
@@ -199,13 +198,11 @@ drag_data_received_cb (GtkWidget        *widget,
 		       guint             time,
 		       Launcher         *launcher)
 {
-	GError *error = NULL;
-	char **envp = NULL;
+	GdkScreen  *screen = launcher_get_screen (launcher);
+	GError     *error = NULL;
+	char      **envp = NULL;
 
-	GdkScreen *screen = launcher_get_screen (launcher);
-
-	if (gdk_screen_get_default () != screen)
-		envp = egg_screen_exec_environment (screen);
+	envp = panel_make_environment_for_screen (screen, NULL);
 
 	gnome_desktop_item_drop_uri_list_with_env (launcher->ditem,
 						   (const char *)selection_data->data,
@@ -696,9 +693,6 @@ void
 launcher_properties (Launcher  *launcher,
 		     GdkScreen *screen)
 {
-	if (panel_profile_get_inhibit_command_line ())
-		return;
-
 	if (launcher->prop_dialog != NULL) {
 		gtk_window_set_screen (
 			GTK_WINDOW (launcher->prop_dialog), screen);
@@ -710,6 +704,16 @@ launcher_properties (Launcher  *launcher,
 	gtk_widget_show_all (launcher->prop_dialog);
 }
 
+static gboolean
+lancher_properties_enabled (void)
+{
+	if (panel_lockdown_get_locked_down () ||
+	    panel_lockdown_get_disable_command_line ())
+		return FALSE;
+
+	return TRUE;
+}
+
 static Launcher *
 load_launcher_applet (const char       *location,
 		      PanelWidget      *panel,
@@ -732,15 +736,11 @@ load_launcher_applet (const char       *location,
 	if (!launcher->info)
 		return NULL;
 
-	if ( ! panel_profile_get_locked_down () &&
-	     ! panel_toplevel_get_locked_down (panel->toplevel) &&
-	     /* if the command line is inhibited, don't allow editting launchers,
-		since that would allow running arbitrary commands */
-	     ! panel_profile_get_inhibit_command_line ())
-		panel_applet_add_callback (launcher->info,
-					   "properties",
-					   GTK_STOCK_PROPERTIES,
-					   _("_Properties"));
+	panel_applet_add_callback (launcher->info,
+				   "properties",
+				   GTK_STOCK_PROPERTIES,
+				   _("_Properties"),
+				   lancher_properties_enabled);
 
 	panel_widget_set_applet_expandable (panel, GTK_WIDGET (launcher->button), FALSE, TRUE);
 	panel_widget_set_applet_size_constrained (panel, GTK_WIDGET (launcher->button), TRUE);
@@ -853,6 +853,7 @@ ensure_item_localefiled (GnomeDesktopItem *ditem, const char *field)
 static void
 really_add_launcher (GtkWidget *dialog, int response, gpointer data)
 {
+	GtkWidget *err_dialog;
 	GnomeDItemEdit *dedit = GNOME_DITEM_EDIT(data);
 	int pos = GPOINTER_TO_INT (g_object_get_data (G_OBJECT (dialog), "pos"));
 	PanelWidget *panel = g_object_get_data (G_OBJECT (dialog), "panel");
@@ -863,6 +864,36 @@ really_add_launcher (GtkWidget *dialog, int response, gpointer data)
 
 		ditem = gnome_ditem_edit_get_ditem (dedit);
 
+		/* check for valid name */
+		if (string_empty (gnome_desktop_item_get_localestring (ditem, GNOME_DESKTOP_ITEM_NAME))) {
+			err_dialog = panel_error_dialog (gtk_window_get_screen (GTK_WINDOW (dialog)),
+						         "cannot_create_launcher",
+						         _("Cannot create launcher"),
+						         _("You have to specify a name."));
+			g_signal_connect_swapped (G_OBJECT (err_dialog),
+						  "destroy",
+						   G_CALLBACK (panel_pop_window_busy),
+						   G_OBJECT (dialog));
+			return;
+		}
+		
+
+		/* check for valid URL or command */
+		if ((gnome_desktop_item_get_entry_type (ditem) == GNOME_DESKTOP_ITEM_TYPE_APPLICATION && 
+		     string_empty (gnome_desktop_item_get_string (ditem, GNOME_DESKTOP_ITEM_EXEC))) ||
+		    (gnome_desktop_item_get_entry_type (ditem) == GNOME_DESKTOP_ITEM_TYPE_LINK &&
+		     string_empty (gnome_desktop_item_get_string (ditem, GNOME_DESKTOP_ITEM_URL)))) {
+			err_dialog = panel_error_dialog (gtk_window_get_screen (GTK_WINDOW (dialog)),
+							 "cannot_create_launcher",
+							 _("Cannot create launcher"),
+							 _("You have to specify a valid URL or command."));
+			g_signal_connect_swapped (G_OBJECT (err_dialog),
+						  "destroy",
+						  G_CALLBACK (panel_pop_window_busy),
+						  G_OBJECT (dialog));
+			return;
+		}
+
 		/* Make sure we set the "C" locale strings to the terms we set
 		   here.  This is so that if the user logs into another locale
 		   they get their own description there rather then empty.  It
