@@ -31,7 +31,6 @@
 #include <libgnome/gnome-i18n.h>
 
 #include "applet.h"
-#include "egg-screen-exec.h"
 #include "menu.h"
 #include "panel-config-global.h"
 #include "panel-gconf.h"
@@ -44,6 +43,7 @@
 #include "panel-globals.h"
 #include "panel-run-dialog.h"
 #include "panel-a11y.h"
+#include "panel-lockdown.h"
 
 #define PANEL_ACTION_BUTTON_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_ACTION_BUTTON, PanelActionButtonPrivate))
 
@@ -82,21 +82,48 @@ panel_action_lock_screen (GtkWidget *widget)
 	panel_lock_screen (gtk_widget_get_screen (widget));
 }
 
+static gboolean
+screensaver_properties_enabled (void)
+{
+	if (panel_lockdown_get_locked_down () ||
+	    panel_lockdown_get_disable_lock_screen ())
+		return FALSE;
+
+	return TRUE;
+}
+
 static void
 panel_action_lock_setup_menu (PanelActionButton *button)
 {
-	panel_applet_add_callback (button->priv->info, "activate", NULL, _("_Activate Screensaver"));
-	panel_applet_add_callback (button->priv->info, "lock",     NULL, _("_Lock Screen"));
+	panel_applet_add_callback (button->priv->info,
+				   "activate",
+				   NULL,
+				   _("_Activate Screensaver"),
+				   panel_lockdown_get_disable_lock_screen);
 
-	/* in commie mode don't allow killing the screensaver */
-	if ( ! panel_profile_get_locked_down ())
-		panel_applet_add_callback (button->priv->info, "exit",     NULL, _("_Kill Screensaver Daemon"));
+	panel_applet_add_callback (button->priv->info,
+				   "lock",
+				   NULL,
+				   _("_Lock Screen"),
+				   panel_lockdown_get_disable_lock_screen);
 
-	panel_applet_add_callback (button->priv->info, "restart",  NULL, _("Restart _Screensaver Daemon"));
+	panel_applet_add_callback (button->priv->info,
+				   "exit",
+				   NULL,
+				   _("_Kill Screensaver Daemon"),
+				   screensaver_properties_enabled);
 
-	/* in commie mode don't allow changing the screensaver properties */
-	if ( ! panel_profile_get_locked_down ())
-		panel_applet_add_callback (button->priv->info, "prefs", NULL, _("_Properties"));
+	panel_applet_add_callback (button->priv->info,
+				   "restart",
+				   NULL,
+				   _("Restart _Screensaver Daemon"),
+				   screensaver_properties_enabled);
+
+	panel_applet_add_callback (button->priv->info,
+				   "prefs",
+				   NULL,
+				   _("_Properties"),
+				   screensaver_properties_enabled);
 }
 
 static void
@@ -105,21 +132,20 @@ panel_action_lock_invoke_menu (PanelActionButton *button,
 {
 	char *command = NULL;
 
-	if (!strcmp (callback_name, "restart"))
-		command = g_strdup ("xscreensaver-command -exit ; xscreensaver &");
-
-	else if (!strcmp (callback_name, "prefs"))
+	if (!strcmp (callback_name, "prefs"))
 		command = g_strdup ("xscreensaver-demo");
 
 	else if (!strcmp (callback_name, "activate") ||
 		 !strcmp (callback_name, "lock") ||
-		 !strcmp (callback_name, "exit"))
+		 !strcmp (callback_name, "exit") ||
+		 !strcmp (callback_name, "restart"))
 		command = g_strdup_printf ("xscreensaver-command -%s", callback_name);
 
 	if (command)
-		egg_screen_execute_shell (
+		gdk_spawn_command_line_on_screen (
 			gtk_widget_get_screen (GTK_WIDGET (button)),
-					       g_get_home_dir (), command);
+			command,
+			NULL);
 
 	g_free (command);
 }
@@ -146,9 +172,6 @@ panel_action_logout (GtkWidget *widget)
 void
 panel_action_run_program (GtkWidget *widget)
 {
-	if (panel_profile_get_inhibit_command_line ())
-		return;
-
 	panel_run_dialog_present (gtk_widget_get_screen (widget));
 }
 
@@ -158,16 +181,22 @@ void
 panel_action_search (GtkWidget *widget)
 {
 	GdkScreen *screen;
+	GError    *error = NULL;
 	char      *argv[2] = {"gnome-search-tool", NULL};
 
 	screen = gtk_widget_get_screen (widget);
 
-	if (egg_screen_execute_async (screen, g_get_home_dir (), 1, argv) < 0)
+	if (!gdk_spawn_on_screen (screen, NULL, argv, NULL,
+				  G_SPAWN_SEARCH_PATH,
+				  NULL, NULL, NULL, &error)) {
 		panel_error_dialog (screen,
 				    "cannot_exec_gnome-search-tool",
 				    _("Cannot execute '%s'"),
-				    NULL,
-				    "gnome-search-tool");
+				    "%s",
+				    "gnome-search-tool",
+				    error->message);
+		g_error_free (error);
+	}
 }
 
 /* Take Screenshot
@@ -176,16 +205,22 @@ void
 panel_action_screenshot (GtkWidget *widget)
 {
 	GdkScreen *screen;
+	GError    *error = NULL;
 	char      *argv [2] = {"gnome-panel-screenshot", NULL};
 
 	screen = gtk_widget_get_screen (widget);
 
-	if (egg_screen_execute_async (screen, g_get_home_dir (), 1, argv) < 0)
+	if (!gdk_spawn_on_screen (screen, NULL, argv, NULL,
+				  G_SPAWN_SEARCH_PATH,
+				  NULL, NULL, NULL, &error)) {
 		panel_error_dialog (screen,
 				    "cannot_exec_gnome-panel-screenshot",
 				    _("Cannot execute '%s'"),
-				    NULL,
-				    "gnome-panel-screenshot");
+				    "%s",
+				    "gnome-panel-screenshot",
+				    error->message);
+		g_error_free (error);
+	}
 }
 
 /* Force Quit
@@ -193,14 +228,7 @@ panel_action_screenshot (GtkWidget *widget)
 static void
 panel_action_force_quit (GtkWidget *widget)
 {
-	GdkScreen *screen;
-
-	if (panel_profile_get_inhibit_force_quit ())
-		return;
-
-	screen = gtk_widget_get_screen (widget);
-
-	panel_force_quit (screen);
+	panel_force_quit (gtk_widget_get_screen (widget));
 }
 
 typedef struct {
@@ -212,6 +240,7 @@ typedef struct {
 	void                  (*setup_menu)  (PanelActionButton *button);
 	void                  (*invoke_menu) (PanelActionButton *button,
 					      const char        *callback_name);
+	gboolean              (*is_disabled) (void);
 } PanelAction;
 
 /* Keep order in sync with PanelActionButtonType
@@ -223,35 +252,47 @@ static PanelAction actions [] = {
 		N_("Lock screen"), "gospanel-21",
 		panel_action_lock_screen,
 		panel_action_lock_setup_menu,
-		panel_action_lock_invoke_menu
+		panel_action_lock_invoke_menu,
+		panel_lockdown_get_disable_lock_screen
 	},
 	{
 		PANEL_ACTION_LOGOUT, PANEL_STOCK_LOGOUT,
 		N_("Log out of GNOME"), "gospanel-20",
-		panel_action_logout, NULL, NULL
+		panel_action_logout, NULL, NULL,
+		panel_lockdown_get_disable_log_out
 	},
 	{
 		PANEL_ACTION_RUN, PANEL_STOCK_RUN,
 		N_("Run Application"), "gospanel-555",
-		panel_action_run_program, NULL, NULL
+		panel_action_run_program, NULL, NULL,
+		panel_lockdown_get_disable_command_line
 	},
 	{
 		PANEL_ACTION_SEARCH, PANEL_STOCK_SEARCHTOOL,
 		N_("Search for Files"), "gospanel-554",
-		panel_action_search, NULL, NULL
+		panel_action_search, NULL, NULL, NULL
 	},
 	{
 		PANEL_ACTION_SCREENSHOT, PANEL_STOCK_SCREENSHOT,
 		N_("Screenshot"), "gospanel-553",
-		panel_action_screenshot, NULL, NULL
+		panel_action_screenshot, NULL, NULL, NULL
 	},
 	{
 		PANEL_ACTION_FORCE_QUIT, PANEL_STOCK_FORCE_QUIT,
 		N_("Force Quit"), "gospanel-563",
-		panel_action_force_quit, NULL, NULL
+		panel_action_force_quit, NULL, NULL,
+		panel_lockdown_get_disable_force_quit
 	}
 };
 
+static void
+panel_action_button_update_sensitivity (PanelActionButton *button)
+{
+	if (actions [button->priv->type].is_disabled)
+		button_widget_set_activatable (BUTTON_WIDGET (button),
+					       !actions [button->priv->type].is_disabled ());
+}
+
 static void
 panel_action_button_finalize (GObject *object)
 {
@@ -260,6 +301,9 @@ panel_action_button_finalize (GObject *object)
 	button->priv->info = NULL;
 	button->priv->type = PANEL_ACTION_NONE;
 
+	panel_lockdown_notify_remove (G_CALLBACK (panel_action_button_update_sensitivity),
+				      button);
+
 	gconf_client_notify_remove (panel_gconf_get_client (),
 				    button->priv->gconf_notify);
 	button->priv->gconf_notify = 0;
@@ -463,6 +507,8 @@ panel_action_button_set_type (PanelActionButton     *button,
 	gtk_tooltips_set_tip (panel_tooltips, GTK_WIDGET (button),
 			      _(actions [type].tooltip), NULL);
 	panel_a11y_set_atk_name_desc (GTK_WIDGET (button), _(actions [type].tooltip), NULL);
+
+	panel_action_button_update_sensitivity (button);
 }
 
 static void
@@ -502,6 +548,9 @@ panel_action_button_connect_to_gconf (PanelActionButton *button)
 		gconf_client_notify_add (panel_gconf_get_client (), key, 
 					 (GConfClientNotifyFunc) panel_action_button_type_changed,
 					 button, NULL, NULL);
+
+	panel_lockdown_notify_add (G_CALLBACK (panel_action_button_update_sensitivity),
+				   button);
 }
 
 static void
@@ -543,8 +592,11 @@ panel_action_button_load (PanelActionButtonType  type,
 		return;
 	}
 
-	panel_applet_add_callback (
-		button->priv->info, "help", GTK_STOCK_HELP, _("_Help"));
+	panel_applet_add_callback (button->priv->info,
+				   "help",
+				   GTK_STOCK_HELP,
+				   _("_Help"),
+				   NULL);
 
 	panel_widget_set_applet_expandable (panel, GTK_WIDGET (button), FALSE, TRUE);
 	panel_widget_set_applet_size_constrained (panel, GTK_WIDGET (button), TRUE);
