@@ -38,52 +38,59 @@
 #include "menu-util.h"
 #include "menu.h"
 #include "quick-desktop-reader.h"
+#include "panel-lockdown.h"
+
+#define PANEL_MENU_BUTTON_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), PANEL_TYPE_MENU_BUTTON, PanelMenuButtonPrivate))
 
 enum {
 	PROP_0,
 	PROP_MENU_PATH,
 	PROP_CUSTOM_ICON,
+	PROP_TOOLTIP,
 	PROP_USE_MENU_PATH,
 	PROP_USE_CUSTOM_ICON,
 	PROP_DND_ENABLED
 };
 
 struct _PanelMenuButtonPrivate {
-	AppletInfo            *info;
 	PanelToplevel         *toplevel;
 	guint                  gconf_notify;
+	char                  *applet_id;
 
 	GtkMenu               *menu;
 
 	char                  *menu_path;
 	char                  *custom_icon;
+	char                  *tooltip;
 
 	guint                  use_menu_path : 1;
 	guint                  use_custom_icon : 1;
 	guint                  dnd_enabled : 1;
 };
 
+static void panel_menu_button_disconnect_from_gconf (PanelMenuButton *button);
+static void panel_menu_button_recreate_menu         (PanelMenuButton *button);
+
 static GObjectClass *parent_class;
 
 static void
 panel_menu_button_instance_init (PanelMenuButton      *button,
 				 PanelMenuButtonClass *klass)
 {
-	button->priv = g_new0 (PanelMenuButtonPrivate, 1);
+	button->priv = PANEL_MENU_BUTTON_GET_PRIVATE (button);
 
-	button->priv->info         = NULL;
+	button->priv->applet_id    = NULL;
 	button->priv->toplevel     = NULL;
 	button->priv->gconf_notify = 0;
 
 	button->priv->menu_path   = NULL;
 	button->priv->custom_icon = NULL;
+	button->priv->tooltip = NULL;
 
 	button->priv->use_menu_path   = FALSE;
 	button->priv->use_custom_icon = FALSE;
 
 	button_widget_set_stock_id (BUTTON_WIDGET (button), PANEL_STOCK_MAIN_MENU);
-
-	gtk_tooltips_set_tip (panel_tooltips, GTK_WIDGET (button), _("Main Menu"), NULL);
 }
 
 static void
@@ -91,10 +98,10 @@ panel_menu_button_finalize (GObject *object)
 {
 	PanelMenuButton *button = PANEL_MENU_BUTTON (object);
 
-	button->priv->info = NULL;
+	panel_lockdown_notify_remove (G_CALLBACK (panel_menu_button_recreate_menu),
+				      button);
 
-	gconf_client_notify_remove (panel_gconf_get_client (), button->priv->gconf_notify);
-	button->priv->gconf_notify = 0;
+	panel_menu_button_disconnect_from_gconf (button);
 
 	if (button->priv->menu) {
 		/* detaching the menu will kill our reference */
@@ -102,14 +109,17 @@ panel_menu_button_finalize (GObject *object)
 		button->priv->menu = NULL;
 	}
 
+	g_free (button->priv->applet_id);
+	button->priv->applet_id = NULL;
+
 	g_free (button->priv->menu_path);
 	button->priv->menu_path = NULL;
 
 	g_free (button->priv->custom_icon);
 	button->priv->custom_icon = NULL;
 
-	g_free (button->priv);
-	button->priv = NULL;
+	g_free (button->priv->tooltip);
+	button->priv->tooltip = NULL;
 
 	parent_class->finalize (object);
 }
@@ -133,6 +143,9 @@ panel_menu_button_get_property (GObject    *object,
 	case PROP_CUSTOM_ICON:
 		g_value_set_string (value, button->priv->custom_icon);
 		break;
+	case PROP_TOOLTIP:
+		g_value_set_string (value, button->priv->tooltip);
+		break;
 	case PROP_USE_MENU_PATH:
 		g_value_set_boolean (value, button->priv->use_menu_path);
 		break;
@@ -167,6 +180,9 @@ panel_menu_button_set_property (GObject      *object,
 	case PROP_CUSTOM_ICON:
                 panel_menu_button_set_custom_icon (button, g_value_get_string (value));
                 break;
+	case PROP_TOOLTIP:
+		panel_menu_button_set_tooltip (button, g_value_get_string (value));
+		break;
 	case PROP_USE_MENU_PATH:
 		panel_menu_button_set_use_menu_path (button, g_value_get_boolean (value));
 		break;
@@ -293,6 +309,14 @@ panel_menu_button_menu_detacher	(GtkWidget *widget,
 	button->priv->menu = NULL;
 }
 
+static void
+panel_menu_button_recreate_menu (PanelMenuButton *button)
+{
+	if (button->priv->menu)
+		gtk_menu_detach (button->priv->menu);
+	button->priv->menu = NULL;
+}
+
 void
 panel_menu_button_popup_menu (PanelMenuButton *button,
 			      guint            n_button,
@@ -339,41 +363,6 @@ panel_menu_button_popup_menu (PanelMenuButton *button,
 			activate_time);
 }
 
-static void
-panel_menu_button_handle_event (PanelMenuButton *button)
-{
-	GdkEvent *event;
-	guint     n_button;
-	guint32   activate_time;
-
-	event = gtk_get_current_event ();
-
-	switch (event->type) {
-	case GDK_BUTTON_PRESS:
-	case GDK_2BUTTON_PRESS:
-	case GDK_3BUTTON_PRESS:
-	case GDK_BUTTON_RELEASE:
-		activate_time = event->button.time;
-		n_button = event->button.button;
-		break;
-	case GDK_KEY_PRESS:
-	case GDK_KEY_RELEASE:
-		activate_time = event->key.time;
-		n_button = 1;
-		break;
-	default:
-		activate_time = n_button = 0;
-		g_assert_not_reached ();
-		break;
-	}
-	
-	gtk_grab_remove (GTK_WIDGET (button));
-
-	panel_menu_button_popup_menu (button, n_button, activate_time);
-
-	gdk_event_free (event);
-}
-
 static void
 panel_menu_button_pressed (GtkButton *gtk_button)
 {
@@ -386,13 +375,14 @@ panel_menu_button_pressed (GtkButton *gtk_button)
 	if (GTK_BUTTON_CLASS (parent_class)->pressed)
 		GTK_BUTTON_CLASS (parent_class)->pressed (gtk_button);
 
-	panel_menu_button_handle_event (button);
+	panel_menu_button_popup_menu (button, 0, gtk_get_current_event_time());
 }
 
 static void
 panel_menu_button_clicked (GtkButton *gtk_button)
 {
 	PanelMenuButton *button;
+	GdkEvent        *event;
 
 	g_return_if_fail (PANEL_IS_MENU_BUTTON (gtk_button));
 
@@ -401,12 +391,18 @@ panel_menu_button_clicked (GtkButton *gtk_button)
 	if (GTK_BUTTON_CLASS (parent_class)->clicked)
 		GTK_BUTTON_CLASS (parent_class)->clicked (gtk_button);
 
-	panel_menu_button_handle_event (button);
+	if ((event = gtk_get_current_event ())) {
+		panel_menu_button_popup_menu (button,
+					      event->button.button,
+					      event->button.time);
+		gdk_event_free (event);
+	} else {
+		panel_menu_button_popup_menu (button, 1, GDK_CURRENT_TIME);
+	}
 }
 
 static void
-panel_menu_button_class_init (PanelMenuButtonClass *klass,
-			   gpointer           dummy)
+panel_menu_button_class_init (PanelMenuButtonClass *klass)
 {
 	GObjectClass   *gobject_class = (GObjectClass   *) klass;
 	GtkWidgetClass *widget_class  = (GtkWidgetClass *) klass;
@@ -424,6 +420,8 @@ panel_menu_button_class_init (PanelMenuButtonClass *klass,
 	button_class->clicked = panel_menu_button_clicked;
 	button_class->pressed = panel_menu_button_pressed;
 
+	g_type_class_add_private (klass, sizeof (PanelMenuButtonPrivate));
+
 	g_object_class_install_property (
 			gobject_class,
 			PROP_MENU_PATH,
@@ -442,6 +440,15 @@ panel_menu_button_class_init (PanelMenuButtonClass *klass,
 					     NULL,
 					     G_PARAM_READWRITE));
 
+	g_object_class_install_property (
+			gobject_class,
+			PROP_TOOLTIP,
+			g_param_spec_string ("tooltip",
+					     _("Tooltip"),
+					     _("Tooltip displayed for the menu"),
+					     NULL,
+					     G_PARAM_READWRITE));
+
 	g_object_class_install_property (
 			gobject_class,
 			PROP_USE_MENU_PATH,
@@ -517,6 +524,10 @@ panel_menu_button_gconf_notify (GConfClient     *client,
 		if (value && value->type == GCONF_VALUE_STRING)
 			panel_menu_button_set_custom_icon (button,
 							   gconf_value_get_string (value));
+	} else if (!strcmp (key, "tooltip")) {
+		if (value && value->type == GCONF_VALUE_STRING)
+			panel_menu_button_set_tooltip (button,
+						       gconf_value_get_string (value));
 	} else if (!strcmp (key, "use_menu_path")) {
 		if (value && value->type == GCONF_VALUE_BOOL)
 			panel_menu_button_set_use_menu_path (button,
@@ -540,7 +551,7 @@ panel_menu_button_connect_to_gconf (PanelMenuButton *button)
 
 	key = panel_gconf_sprintf (PANEL_CONFIG_DIR "/%s/objects/%s",
 				   profile,
-				   button->priv->info->id);
+				   button->priv->applet_id);
 	gconf_client_add_dir (client, key, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
 	button->priv->gconf_notify =
 		gconf_client_notify_add (client, key,
@@ -548,11 +559,35 @@ panel_menu_button_connect_to_gconf (PanelMenuButton *button)
 					 button, NULL, NULL);
 }
 
+static void
+panel_menu_button_disconnect_from_gconf (PanelMenuButton *button)
+{
+	GConfClient *client;
+	const char  *key;
+	const char  *profile;
+
+	if (!button->priv->gconf_notify)
+		return;
+
+	client  = panel_gconf_get_client ();
+	profile = panel_profile_get_name ();
+
+	key = panel_gconf_sprintf (PANEL_CONFIG_DIR "/%s/objects/%s",
+				   profile,
+				   button->priv->applet_id);
+
+	gconf_client_notify_remove (client, button->priv->gconf_notify);
+	button->priv->gconf_notify = 0;
+
+	gconf_client_remove_dir (client, key, NULL);
+}
+
 static void
 panel_menu_button_load (const char  *menu_path,
 			gboolean     use_menu_path,
 			const char  *custom_icon,
 			gboolean     use_custom_icon,
+			const char  *tooltip,
 			PanelWidget *panel,
 			gboolean     locked,
 			int          position,
@@ -560,33 +595,38 @@ panel_menu_button_load (const char  *menu_path,
 			const char  *id)
 {
 	PanelMenuButton *button;
+	AppletInfo      *info;
 
 	g_return_if_fail (panel != NULL);
 
 	button = g_object_new (PANEL_TYPE_MENU_BUTTON,
 			       "menu-path", menu_path,
 			       "custom-icon", custom_icon,
+			       "tooltip", tooltip,
 			       "use-menu-path", use_menu_path,
 			       "use-custom-icon", use_custom_icon,
 			       "has-arrow", TRUE,
 			       NULL);
 
-	button->priv->info = panel_applet_register (
-					GTK_WIDGET (button), NULL, NULL,
-					panel, locked, position, exactpos,
-					PANEL_OBJECT_MENU, id);
-	if (!button->priv->info) {
+	info = panel_applet_register (GTK_WIDGET (button), NULL, NULL,
+				      panel, locked, position, exactpos,
+				      PANEL_OBJECT_MENU, id);
+	if (!info) {
 		gtk_widget_destroy (GTK_WIDGET (button));
 		return;
 	}
 
-	panel_applet_add_callback (
-		button->priv->info, "help", GTK_STOCK_HELP, _("_Help"));
+	button->priv->applet_id = g_strdup (info->id);
+
+	panel_applet_add_callback (info, "help", GTK_STOCK_HELP, _("_Help"), NULL);
 
 	panel_widget_set_applet_expandable (panel, GTK_WIDGET (button), FALSE, TRUE);
 	panel_widget_set_applet_size_constrained (panel, GTK_WIDGET (button), TRUE);
 
 	panel_menu_button_connect_to_gconf (button);
+
+	panel_lockdown_notify_add (G_CALLBACK (panel_menu_button_recreate_menu),
+				   button);
 }
 
 static char *
@@ -673,6 +713,22 @@ panel_menu_button_set_custom_icon (PanelMenuButton *button,
 	panel_menu_button_set_icon (button);
 }
 
+void
+panel_menu_button_set_tooltip (PanelMenuButton *button,
+			       const char      *tooltip)
+{
+	g_return_if_fail (PANEL_IS_MENU_BUTTON (button));
+
+	g_free (button->priv->tooltip);
+	button->priv->tooltip = NULL;
+
+	if (tooltip && tooltip [0]) {
+		button->priv->tooltip = g_strdup (tooltip);
+		gtk_tooltips_set_tip (panel_tooltips, GTK_WIDGET (button),
+				      tooltip, NULL);
+	}
+}
+
 void
 panel_menu_button_set_use_menu_path (PanelMenuButton *button,
 				     gboolean         use_menu_path)
@@ -724,21 +780,22 @@ panel_menu_button_load_from_gconf (PanelWidget *panel,
 	const char  *key;
 	char        *menu_path;
 	char        *custom_icon;
+	char        *tooltip;
 	gboolean     use_menu_path;
 	gboolean     use_custom_icon;
 
 	client  = panel_gconf_get_client ();
 	profile = panel_profile_get_name ();
 
-	key = panel_gconf_sprintf (PANEL_CONFIG_DIR "/%s/objects/%s", profile, id);
-	gconf_client_add_dir (client, key, GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
-
 	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, profile, id, "menu_path");
 	menu_path = gconf_client_get_string (client, key, NULL);
 
 	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, profile, id, "custom_icon");
 	custom_icon = gconf_client_get_string (client, key, NULL);
 
+	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, profile, id, "tooltip");
+	tooltip = gconf_client_get_string (client, key, NULL);
+
 	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, profile, id, "use_menu_path");
 	use_menu_path = gconf_client_get_bool (client, key, NULL);
 
@@ -749,6 +806,7 @@ panel_menu_button_load_from_gconf (PanelWidget *panel,
 				use_menu_path,
 				custom_icon,
 				use_custom_icon,
+				tooltip,
 				panel,
 				locked,
 				position,
@@ -757,13 +815,15 @@ panel_menu_button_load_from_gconf (PanelWidget *panel,
 
 	g_free (menu_path);
 	g_free (custom_icon);
+	g_free (tooltip);
 }
 
 void
 panel_menu_button_create (PanelToplevel *toplevel,
 			  int            position,
 			  const char    *menu_path,
-			  gboolean       use_menu_path)
+			  gboolean       use_menu_path,
+			  const char    *tooltip)
 {
 	GConfClient *client;
 	const char  *profile;
@@ -783,6 +843,9 @@ panel_menu_button_create (PanelToplevel *toplevel,
 		gconf_client_set_string (client, key, menu_path, NULL);
 	}
 
+	key = panel_gconf_full_key (PANEL_GCONF_OBJECTS, profile, id, "tooltip");
+	gconf_client_set_string (client, key, tooltip, NULL);
+
 	/* frees id */
 	panel_profile_add_to_list (PANEL_GCONF_OBJECTS, id);
 }
