@@ -43,8 +43,6 @@
 #include <libgnomevfs/gnome-vfs-utils.h>
 #include <gconf/gconf-client.h>
 
-#include "egg-screen-exec.h"
-
 #include "nothing.h"
 #include "panel-gconf.h"
 #include "panel-util.h"
@@ -55,6 +53,7 @@
 #include "panel-multiscreen.h"
 #include "menu.h"
 #include "menu-fentry.h"
+#include "panel-lockdown.h"
 
 #define ICON_SIZE 48.0
 
@@ -76,8 +75,6 @@ typedef struct {
 	
 	GtkListStore     *program_list_store;
 
-	GtkFileSelection *file_sel;
-	
 	GList            *executables;
 	GCompletion      *completion;
 	
@@ -110,10 +107,6 @@ panel_run_dialog_destroy (PanelRunDialog *dialog)
 {
 	GList *l;
 
-	if (dialog->file_sel != NULL)
-		gtk_widget_destroy (GTK_WIDGET (dialog->file_sel));
-	dialog->file_sel = NULL;
-
 	g_object_unref (dialog->program_list_box);
 	
 	g_slist_foreach (dialog->add_icon_paths, (GFunc) gtk_tree_path_free, NULL);
@@ -261,7 +254,6 @@ panel_run_dialog_launch_command (PanelRunDialog *dialog,
 	gboolean    result;	
 	GError     *error = NULL;
 	char      **argv;
-	char      **envp;
 	int         argc;
 	
 	if (!command_is_executable (command))
@@ -274,19 +266,19 @@ panel_run_dialog_launch_command (PanelRunDialog *dialog,
 	argv [2] = g_strdup (command);
 	
 	screen = gtk_window_get_screen (GTK_WINDOW (dialog->run_dialog));	
-	envp = egg_screen_exec_environment (screen);
 		
 	if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (dialog->terminal_checkbox)))
 		gnome_prepend_terminal_to_vector (&argc, &argv);
 		   
-	result = g_spawn_async (NULL, /* working directory */
-				argv,
-				envp,
-				G_SPAWN_SEARCH_PATH,
-				NULL, /* child setup func */
-				NULL, /* user data */
-				NULL, /* child pid */
-				&error);
+	result = gdk_spawn_on_screen (screen,
+				      NULL, /* working directory */
+				      argv,
+				      NULL, /* envp */
+				      G_SPAWN_SEARCH_PATH,
+				      NULL, /* child setup func */
+				      NULL, /* user data */
+				      NULL, /* child pid */
+				      &error);
 			
 	if (!result) {
 		panel_error_dialog (screen, "cannot_spawn_command",
@@ -299,7 +291,6 @@ panel_run_dialog_launch_command (PanelRunDialog *dialog,
 	}
 				
 	g_strfreev (argv);
-	g_strfreev (envp);
 	
 	return result;
 }
@@ -309,17 +300,12 @@ panel_run_dialog_show_url (PanelRunDialog *dialog,
 	                   const char     *url,
 	                   const char     *escaped)
 {
-	GError     *error = NULL;
-	GdkScreen  *screen;
-	char      **envp;
+	GError    *error = NULL;
+	GdkScreen *screen;
 	
 	screen = gtk_window_get_screen (GTK_WINDOW (dialog->run_dialog));
-	envp = egg_screen_exec_environment (screen);
-	
-	gnome_url_show_with_env (url, envp, &error);
-
-	g_strfreev (envp);
 
+	gnome_url_show_on_screen (url, screen, &error);
 	if (error) {
 		panel_error_dialog (screen, "cannot_show_url",
 				    _("Cannot display location '%s'"),
@@ -338,6 +324,7 @@ static void
 panel_run_dialog_execute (PanelRunDialog *dialog)
 {
 	GdkScreen *screen;
+	GError    *error;
 	gboolean   result;
 	char      *command;
 	char      *escaped;
@@ -368,7 +355,20 @@ panel_run_dialog_execute (PanelRunDialog *dialog)
 		return;
 	}
 		
-	disk = g_locale_from_utf8 (command, -1, NULL, NULL, NULL);
+	error = NULL;
+	disk = g_locale_from_utf8 (command, -1, NULL, NULL, &error);
+
+	if (!disk || error) {
+		panel_error_dialog (screen, "cannot_convert_command_from_utf8",
+				    _("Cannot convert '%s' from UTF-8"),
+				    "%s",
+				    command,
+				    error->message);
+
+		g_error_free (error);
+		return;
+	}
+
 	url = gnome_vfs_make_uri_from_input_with_dirs (disk,
 						       GNOME_VFS_MAKE_URI_DIR_HOMEDIR);
 	escaped = g_markup_escape_text (url, -1);
@@ -949,10 +949,11 @@ panel_run_dialog_update_content (PanelRunDialog *dialog,
 		if (dialog->list_expander->parent)
 			gtk_container_remove (GTK_CONTAINER (dialog->list_expander->parent),
 					      dialog->list_expander);
+
+		gtk_window_set_resizable (GTK_WINDOW (dialog->run_dialog), FALSE);
+                gtk_widget_grab_focus (dialog->gtk_entry);
 		
-	}
-	
-	if (show_list) {
+	} else if (show_list) {
 		gtk_window_resize (GTK_WINDOW (dialog->run_dialog), 100, 300);
 		gtk_window_set_resizable (GTK_WINDOW (dialog->run_dialog), TRUE);
 		gtk_widget_grab_focus (dialog->program_list);
@@ -1014,53 +1015,46 @@ panel_run_dialog_setup_list_expander (PanelRunDialog *dialog,
 }
 
 static void
-file_button_browse_ok (GtkWidget      *button,
-		       PanelRunDialog *dialog)
+file_button_browse_response (GtkWidget      *chooser,
+			     gint            response,
+			     PanelRunDialog *dialog)
 {
-	const char *file;
+	char *file;
 	
-	file = gtk_file_selection_get_filename (dialog->file_sel);
-	panel_run_dialog_append_file (dialog, file);
-	
-	gtk_widget_destroy (GTK_WIDGET (dialog->file_sel));
-	dialog->file_sel = NULL;
+	if (response == GTK_RESPONSE_OK) {
+		file = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (chooser));
+		panel_run_dialog_append_file (dialog, file);
+		g_free (file);
+	}
 
-	gtk_widget_grab_focus (dialog->gtk_entry);
-}
-
-static void
-file_button_browse_cancel (GtkWidget      *button,
-			   PanelRunDialog *dialog)
-{
-	gtk_widget_destroy (GTK_WIDGET (dialog->file_sel));
-	dialog->file_sel = NULL;
-
-	gtk_widget_grab_focus (dialog->gtk_entry);
+	gtk_widget_destroy (chooser);
+ 
+ 	gtk_widget_grab_focus (dialog->gtk_entry);
 }
 
 static void
 file_button_clicked (GtkButton      *button,
 		     PanelRunDialog *dialog)
 {
-	char *home;
+	GtkWidget *chooser;
 
-	dialog->file_sel = GTK_FILE_SELECTION (gtk_file_selection_new (
-						       _("Choose a file to append to the command...")));
+	chooser = gtk_file_chooser_dialog_new (_("Choose a file to append to the command..."),
+					       GTK_WINDOW (dialog->run_dialog),
+					       GTK_FILE_CHOOSER_ACTION_OPEN,
+					       GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
+					       GTK_STOCK_OK, GTK_RESPONSE_OK,
+					       NULL);
 	
-	home = g_strconcat (g_get_home_dir (), "/", NULL);
-	gtk_file_selection_set_filename (dialog->file_sel, home);
-	g_free (home);
+	gtk_file_chooser_set_current_folder (GTK_FILE_CHOOSER (chooser),
+					     g_get_home_dir ());
 	
-	gtk_window_set_transient_for (GTK_WINDOW (dialog->file_sel),
-				      GTK_WINDOW (dialog->run_dialog));
-				      
-	g_signal_connect (dialog->file_sel->ok_button, "clicked",
-			  G_CALLBACK (file_button_browse_ok), dialog);
-			  
-	g_signal_connect (dialog->file_sel->cancel_button, "clicked",
-		 	  G_CALLBACK (file_button_browse_cancel), dialog);
+	gtk_dialog_set_default_response (GTK_DIALOG (chooser), GTK_RESPONSE_OK);
+	gtk_window_set_destroy_with_parent (GTK_WINDOW (chooser), TRUE);
 
-	gtk_window_present (GTK_WINDOW (dialog->file_sel));
+	g_signal_connect (chooser, "response",
+			  G_CALLBACK (file_button_browse_response), dialog);
+
+	gtk_window_present (GTK_WINDOW (chooser));
 }
 
 static void
@@ -1081,6 +1075,7 @@ panel_run_dialog_fill_executables_from (PanelRunDialog *dialog,
 	struct dirent *dent;
 	DIR           *dir;
 	char          *file;
+	struct stat    file_stat;
 
 	dir = opendir (dirname);
 
@@ -1090,7 +1085,8 @@ panel_run_dialog_fill_executables_from (PanelRunDialog *dialog,
 	while ((dent = readdir (dir))) {
 		file = g_build_filename (dirname, dent->d_name, NULL);
 
-		if (!access (file, X_OK))
+		if (!access (file, X_OK) &&
+		    !stat (file, &file_stat) && !S_ISDIR (file_stat.st_mode))
 			dialog->executables = g_list_prepend (dialog->executables,
 							      g_strdup (dent->d_name));
 		
@@ -1475,7 +1471,7 @@ panel_run_dialog_present (GdkScreen *screen)
 	GladeXML *gui;
 	char     *glade_file;
 
-	if (panel_profile_get_inhibit_command_line ())
+	if (panel_lockdown_get_disable_command_line ())
 		return;
 
 	if (static_dialog) {
